//////////////////////////////////////////////////////////////////////////////
// Name:        Elements_CopyConstructors.cpp
// Author:      Alex Thuering
// Copyright:   (c) 2005 Alex Thuering
// Licence:     wxWindows licence
// Notes:       generated by generate.py
//////////////////////////////////////////////////////////////////////////////

#include "SVGCanvasItem.h"
#include "SVGCircleElement.h"
#include "SVGEllipseElement.h"
#include "SVGImageElement.h"
#include "SVGLineElement.h"
#include "SVGPathElement.h"
#include "SVGPolygonElement.h"
#include "SVGPolylineElement.h"
#include "SVGRectElement.h"
#include "SVGTextElement.h"
#include "SVGVideoElement.h"

// wxSVGCircleElement
wxSVGCircleElement::wxSVGCircleElement(wxSVGCircleElement& src):
  wxSVGElement(src),
  wxSVGTests(src),
  wxSVGLangSpace(src),
  wxSVGExternalResourcesRequired(src),
  wxSVGStylable(src),
  wxSVGTransformable(src),
  wxEventTarget(src)
{
  m_cx = src.m_cx;
  m_cy = src.m_cy;
  m_r = src.m_r;
  m_canvasItem = NULL;
}

wxSVGCircleElement::~wxSVGCircleElement()
{
  if (m_canvasItem)
    delete m_canvasItem;
}

// wxSVGEllipseElement
wxSVGEllipseElement::wxSVGEllipseElement(wxSVGEllipseElement& src):
  wxSVGElement(src),
  wxSVGTests(src),
  wxSVGLangSpace(src),
  wxSVGExternalResourcesRequired(src),
  wxSVGStylable(src),
  wxSVGTransformable(src),
  wxEventTarget(src)
{
  m_cx = src.m_cx;
  m_cy = src.m_cy;
  m_rx = src.m_rx;
  m_ry = src.m_ry;
  m_canvasItem = NULL;
}

wxSVGEllipseElement::~wxSVGEllipseElement()
{
  if (m_canvasItem)
    delete m_canvasItem;
}

// wxSVGImageElement
wxSVGImageElement::wxSVGImageElement(wxSVGImageElement& src):
  wxSVGElement(src),
  wxSVGURIReference(src),
  wxSVGTests(src),
  wxSVGLangSpace(src),
  wxSVGExternalResourcesRequired(src),
  wxSVGStylable(src),
  wxSVGTransformable(src),
  wxEventTarget(src)
{
  m_x = src.m_x;
  m_y = src.m_y;
  m_width = src.m_width;
  m_height = src.m_height;
  m_preserveAspectRatio = src.m_preserveAspectRatio;
  m_canvasItem = NULL;
}

wxSVGImageElement::~wxSVGImageElement()
{
  if (m_canvasItem)
    delete m_canvasItem;
}

// wxSVGLineElement
wxSVGLineElement::wxSVGLineElement(wxSVGLineElement& src):
  wxSVGElement(src),
  wxSVGTests(src),
  wxSVGLangSpace(src),
  wxSVGExternalResourcesRequired(src),
  wxSVGStylable(src),
  wxSVGTransformable(src),
  wxEventTarget(src)
{
  m_x1 = src.m_x1;
  m_y1 = src.m_y1;
  m_x2 = src.m_x2;
  m_y2 = src.m_y2;
  m_canvasItem = NULL;
}

wxSVGLineElement::~wxSVGLineElement()
{
  if (m_canvasItem)
    delete m_canvasItem;
}

// wxSVGPathElement
wxSVGPathElement::wxSVGPathElement(wxSVGPathElement& src):
  wxSVGElement(src),
  wxSVGTests(src),
  wxSVGLangSpace(src),
  wxSVGExternalResourcesRequired(src),
  wxSVGStylable(src),
  wxSVGTransformable(src),
  wxEventTarget(src),
  wxSVGAnimatedPathData(src)
{
  m_pathLength = src.m_pathLength;
  m_canvasItem = NULL;
}

wxSVGPathElement::~wxSVGPathElement()
{
  if (m_canvasItem)
    delete m_canvasItem;
}

// wxSVGPolygonElement
wxSVGPolygonElement::wxSVGPolygonElement(wxSVGPolygonElement& src):
  wxSVGElement(src),
  wxSVGTests(src),
  wxSVGLangSpace(src),
  wxSVGExternalResourcesRequired(src),
  wxSVGStylable(src),
  wxSVGTransformable(src),
  wxEventTarget(src),
  wxSVGAnimatedPoints(src)
{
  m_canvasItem = NULL;
}

wxSVGPolygonElement::~wxSVGPolygonElement()
{
  if (m_canvasItem)
    delete m_canvasItem;
}

// wxSVGPolylineElement
wxSVGPolylineElement::wxSVGPolylineElement(wxSVGPolylineElement& src):
  wxSVGElement(src),
  wxSVGTests(src),
  wxSVGLangSpace(src),
  wxSVGExternalResourcesRequired(src),
  wxSVGStylable(src),
  wxSVGTransformable(src),
  wxEventTarget(src),
  wxSVGAnimatedPoints(src)
{
  m_canvasItem = NULL;
}

wxSVGPolylineElement::~wxSVGPolylineElement()
{
  if (m_canvasItem)
    delete m_canvasItem;
}

// wxSVGRectElement
wxSVGRectElement::wxSVGRectElement(wxSVGRectElement& src):
  wxSVGElement(src),
  wxSVGTests(src),
  wxSVGLangSpace(src),
  wxSVGExternalResourcesRequired(src),
  wxSVGStylable(src),
  wxSVGTransformable(src),
  wxEventTarget(src)
{
  m_x = src.m_x;
  m_y = src.m_y;
  m_width = src.m_width;
  m_height = src.m_height;
  m_rx = src.m_rx;
  m_ry = src.m_ry;
  m_canvasItem = NULL;
}

wxSVGRectElement::~wxSVGRectElement()
{
  if (m_canvasItem)
    delete m_canvasItem;
}

// wxSVGTextElement
wxSVGTextElement::wxSVGTextElement(wxSVGTextElement& src):
  wxSVGTextPositioningElement(src),
  wxSVGTransformable(src)
{
  m_canvasItem = NULL;
}

wxSVGTextElement::~wxSVGTextElement()
{
  if (m_canvasItem)
    delete m_canvasItem;
}

// wxSVGVideoElement
wxSVGVideoElement::wxSVGVideoElement(wxSVGVideoElement& src):
  wxSVGElement(src),
  wxSVGURIReference(src),
  wxSVGLangSpace(src),
  wxSVGStylable(src),
  wxSVGTransformable(src)
{
  m_x = src.m_x;
  m_y = src.m_y;
  m_width = src.m_width;
  m_height = src.m_height;
  m_preserveAspectRatio = src.m_preserveAspectRatio;
  m_begin = src.m_begin;
  m_dur = src.m_dur;
  m_clipBegin = src.m_clipBegin;
  m_clipEnd = src.m_clipEnd;
  m_canvasItem = NULL;
}

wxSVGVideoElement::~wxSVGVideoElement()
{
  if (m_canvasItem)
    delete m_canvasItem;
}

