//////////////////////////////////////////////////////////////////////////////
// Name:        css_properties.cpp
// Author:      Alex Thuering
// Copyright:   (c) 2005 Alex Thuering
// Licence:     wxWindows licence
// Notes:       generated by genCSS.py
//////////////////////////////////////////////////////////////////////////////


wxString s_cssPropertyStrings[] = 
{
  wxT("alignment-baseline"),
  wxT("baseline-shift"),
  wxT("clip"),
  wxT("clip-path"),
  wxT("clip-rule"),
  wxT("color"),
  wxT("color-interpolation"),
  wxT("color-interpolation-filters"),
  wxT("color-profile"),
  wxT("color-rendering"),
  wxT("cursor"),
  wxT("direction"),
  wxT("display"),
  wxT("dominant-baseline"),
  wxT("enable-background"),
  wxT("fill"),
  wxT("fill-opacity"),
  wxT("fill-rule"),
  wxT("filter"),
  wxT("flood-color"),
  wxT("flood-opacity"),
  wxT("font-family"),
  wxT("font-size"),
  wxT("font-size-adjust"),
  wxT("font-stretch"),
  wxT("font-style"),
  wxT("font-variant"),
  wxT("font-weight"),
  wxT("glyph-orientation-horizontal"),
  wxT("glyph-orientation-vertical"),
  wxT("image-rendering"),
  wxT("kerning"),
  wxT("letter-spacing"),
  wxT("lighting-color"),
  wxT("marker-end"),
  wxT("marker-mid"),
  wxT("marker-start"),
  wxT("mask"),
  wxT("opacity"),
  wxT("overflow"),
  wxT("pointer-events"),
  wxT("shape-rendering"),
  wxT("stop-color"),
  wxT("stop-opacity"),
  wxT("stroke"),
  wxT("stroke-dasharray"),
  wxT("stroke-dashoffset"),
  wxT("stroke-linecap"),
  wxT("stroke-linejoin"),
  wxT("stroke-miterlimit"),
  wxT("stroke-opacity"),
  wxT("stroke-width"),
  wxT("text-anchor"),
  wxT("text-decoration"),
  wxT("text-rendering"),
  wxT("unicode-bidi"),
  wxT("visibility"),
  wxT("word-spacing"),
  wxT("writing-mode")
};
    
