//////////////////////////////////////////////////////////////////////////////
// Name:        css_colors.cpp
// Purpose:     
// Author:      Alex Thuering
// Created:     2005/05/04
// RCS-ID:      $Id: css_colors.cpp,v 1.1 2005/06/09 00:24:35 ntalex Exp $
// Copyright:   (c) 2005 Alex Thuering
// Licence:     wxWindows licence
//////////////////////////////////////////////////////////////////////////////

struct wxCSSNamedColour
{
  wxString name;
  wxRGBColor colour;
};

wxCSSNamedColour s_cssNamedColors[] = 
{
  { wxT("aliceblue"), wxRGBColor(240, 248, 255) }, 
  { wxT("antiquewhite"), wxRGBColor(250, 235, 215) }, 
  { wxT("aqua"), wxRGBColor(0, 255, 255) }, 
  { wxT("aquamarine"), wxRGBColor(127, 255, 212) }, 
  { wxT("azure"), wxRGBColor(240, 255, 255) }, 
  { wxT("beige"), wxRGBColor(245, 245, 220) }, 
  { wxT("bisque"), wxRGBColor(255, 228, 196) }, 
  { wxT("black"), wxRGBColor(0, 0, 0) }, 
  { wxT("blanchedalmond"), wxRGBColor(255, 235, 205) }, 
  { wxT("blue"), wxRGBColor(0, 0, 255) }, 
  { wxT("blueviolet"), wxRGBColor(138, 43, 226) }, 
  { wxT("brown"), wxRGBColor(165, 42, 42) }, 
  { wxT("burlywood"), wxRGBColor(222, 184, 135) }, 
  { wxT("cadetblue"), wxRGBColor(95, 158, 160) }, 
  { wxT("chartreuse"), wxRGBColor(127, 255, 0) }, 
  { wxT("chocolate"), wxRGBColor(210, 105, 30) }, 
  { wxT("coral"), wxRGBColor(255, 127, 80) }, 
  { wxT("cornflowerblue"), wxRGBColor(100, 149, 237) }, 
  { wxT("cornsilk"), wxRGBColor(255, 248, 220) }, 
  { wxT("crimson"), wxRGBColor(220, 20, 60) }, 
  { wxT("cyan"), wxRGBColor(0, 255, 255) }, 
  { wxT("darkblue"), wxRGBColor(0, 0, 139) }, 
  { wxT("darkcyan"), wxRGBColor(0, 139, 139) }, 
  { wxT("darkgoldenrod"), wxRGBColor(184, 134, 11) }, 
  { wxT("darkgray"), wxRGBColor(169, 169, 169) }, 
  { wxT("darkgreen"), wxRGBColor(0, 100, 0) }, 
  { wxT("darkgrey"), wxRGBColor(169, 169, 169) }, 
  { wxT("darkkhaki"), wxRGBColor(189, 183, 107) }, 
  { wxT("darkmagenta"), wxRGBColor(139, 0, 139) }, 
  { wxT("darkolivegreen"), wxRGBColor(85, 107, 47) }, 
  { wxT("darkorange"), wxRGBColor(255, 140, 0) }, 
  { wxT("darkorchid"), wxRGBColor(153, 50, 204) }, 
  { wxT("darkred"), wxRGBColor(139, 0, 0) }, 
  { wxT("darksalmon"), wxRGBColor(233, 150, 122) }, 
  { wxT("darkseagreen"), wxRGBColor(143, 188, 143) }, 
  { wxT("darkslateblue"), wxRGBColor(72, 61, 139) }, 
  { wxT("darkslategray"), wxRGBColor(47, 79, 79) }, 
  { wxT("darkslategrey"), wxRGBColor(47, 79, 79) }, 
  { wxT("darkturquoise"), wxRGBColor(0, 206, 209) }, 
  { wxT("darkviolet"), wxRGBColor(148, 0, 211) }, 
  { wxT("deeppink"), wxRGBColor(255, 20, 147) }, 
  { wxT("deepskyblue"), wxRGBColor(0, 191, 255) }, 
  { wxT("dimgray"), wxRGBColor(105, 105, 105) }, 
  { wxT("dimgrey"), wxRGBColor(105, 105, 105) }, 
  { wxT("dodgerblue"), wxRGBColor(30, 144, 255) }, 
  { wxT("firebrick"), wxRGBColor(178, 34, 34) }, 
  { wxT("floralwhite"), wxRGBColor(255, 250, 240) }, 
  { wxT("forestgreen"), wxRGBColor(34, 139, 34) }, 
  { wxT("fuchsia"), wxRGBColor(255, 0, 255) }, 
  { wxT("gainsboro"), wxRGBColor(220, 220, 220) }, 
  { wxT("ghostwhite"), wxRGBColor(248, 248, 255) }, 
  { wxT("gold"), wxRGBColor(255, 215, 0) }, 
  { wxT("goldenrod"), wxRGBColor(218, 165, 32) }, 
  { wxT("gray"), wxRGBColor(128, 128, 128) }, 
  { wxT("green"), wxRGBColor(0, 128, 0) }, 
  { wxT("greenyellow"), wxRGBColor(173, 255, 47) }, 
  { wxT("grey"), wxRGBColor(128, 128, 128) }, 
  { wxT("honeydew"), wxRGBColor(240, 255, 240) }, 
  { wxT("hotpink"), wxRGBColor(255, 105, 180) }, 
  { wxT("indianred"), wxRGBColor(205, 92, 92) }, 
  { wxT("indigo"), wxRGBColor(75, 0, 130) }, 
  { wxT("ivory"), wxRGBColor(255, 255, 240) }, 
  { wxT("khaki"), wxRGBColor(240, 230, 140) }, 
  { wxT("lavender"), wxRGBColor(230, 230, 250) }, 
  { wxT("lavenderblush"), wxRGBColor(255, 240, 245) }, 
  { wxT("lawngreen"), wxRGBColor(124, 252, 0) }, 
  { wxT("lemonchiffon"), wxRGBColor(255, 250, 205) }, 
  { wxT("lightblue"), wxRGBColor(173, 216, 230) }, 
  { wxT("lightcoral"), wxRGBColor(240, 128, 128) }, 
  { wxT("lightcyan"), wxRGBColor(224, 255, 255) }, 
  { wxT("lightgoldenrodyellow"), wxRGBColor(250, 250, 210) }, 
  { wxT("lightgray"), wxRGBColor(211, 211, 211) }, 
  { wxT("lightgreen"), wxRGBColor(144, 238, 144) }, 
  { wxT("lightgrey"), wxRGBColor(211, 211, 211) }, 
  { wxT("lightpink"), wxRGBColor(255, 182, 193) }, 
  { wxT("lightsalmon"), wxRGBColor(255, 160, 122) }, 
  { wxT("lightseagreen"), wxRGBColor(32, 178, 170) }, 
  { wxT("lightskyblue"), wxRGBColor(135, 206, 250) }, 
  { wxT("lightslategray"), wxRGBColor(119, 136, 153) }, 
  { wxT("lightslategrey"), wxRGBColor(119, 136, 153) }, 
  { wxT("lightsteelblue"), wxRGBColor(176, 196, 222) }, 
  { wxT("lightyellow"), wxRGBColor(255, 255, 224) }, 
  { wxT("lime"), wxRGBColor(0, 255, 0) }, 
  { wxT("limegreen"), wxRGBColor(50, 205, 50) }, 
  { wxT("linen"), wxRGBColor(250, 240, 230) }, 
  { wxT("magenta"), wxRGBColor(255, 0, 255) }, 
  { wxT("maroon"), wxRGBColor(128, 0, 0) }, 
  { wxT("mediumaquamarine"), wxRGBColor(102, 205, 170) }, 
  { wxT("mediumblue"), wxRGBColor(0, 0, 205) }, 
  { wxT("mediumorchid"), wxRGBColor(186, 85, 211) }, 
  { wxT("mediumpurple"), wxRGBColor(147, 112, 219) }, 
  { wxT("mediumseagreen"), wxRGBColor(60, 179, 113) }, 
  { wxT("mediumslateblue"), wxRGBColor(123, 104, 238) }, 
  { wxT("mediumspringgreen"), wxRGBColor(0, 250, 154) }, 
  { wxT("mediumturquoise"), wxRGBColor(72, 209, 204) }, 
  { wxT("mediumvioletred"), wxRGBColor(199, 21, 133) }, 
  { wxT("midnightblue"), wxRGBColor(25, 25, 112) }, 
  { wxT("mintcream"), wxRGBColor(245, 255, 250) }, 
  { wxT("mistyrose"), wxRGBColor(255, 228, 225) }, 
  { wxT("moccasin"), wxRGBColor(255, 228, 181) }, 
  { wxT("navajowhite"), wxRGBColor(255, 222, 173) }, 
  { wxT("navy"), wxRGBColor(0, 0, 128) }, 
  { wxT("oldlace"), wxRGBColor(253, 245, 230) }, 
  { wxT("olive"), wxRGBColor(128, 128, 0) }, 
  { wxT("olivedrab"), wxRGBColor(107, 142, 35) }, 
  { wxT("orange"), wxRGBColor(255, 165, 0) }, 
  { wxT("orangered"), wxRGBColor(255, 69, 0) }, 
  { wxT("orchid"), wxRGBColor(218, 112, 214) }, 
  { wxT("palegoldenrod"), wxRGBColor(238, 232, 170) }, 
  { wxT("palegreen"), wxRGBColor(152, 251, 152) }, 
  { wxT("paleturquoise"), wxRGBColor(175, 238, 238) }, 
  { wxT("palevioletred"), wxRGBColor(219, 112, 147) }, 
  { wxT("papayawhip"), wxRGBColor(255, 239, 213) }, 
  { wxT("peachpuff"), wxRGBColor(255, 218, 185) }, 
  { wxT("peru"), wxRGBColor(205, 133, 63) }, 
  { wxT("pink"), wxRGBColor(255, 192, 203) }, 
  { wxT("plum"), wxRGBColor(221, 160, 221) }, 
  { wxT("powderblue"), wxRGBColor(176, 224, 230) }, 
  { wxT("purple"), wxRGBColor(128, 0, 128) }, 
  { wxT("red"), wxRGBColor(255, 0, 0) }, 
  { wxT("rosybrown"), wxRGBColor(188, 143, 143) }, 
  { wxT("royalblue"), wxRGBColor(65, 105, 225) }, 
  { wxT("saddlebrown"), wxRGBColor(139, 69, 19) }, 
  { wxT("salmon"), wxRGBColor(250, 128, 114) }, 
  { wxT("sandybrown"), wxRGBColor(244, 164, 96) }, 
  { wxT("seagreen"), wxRGBColor(46, 139, 87) }, 
  { wxT("seashell"), wxRGBColor(255, 245, 238) }, 
  { wxT("sienna"), wxRGBColor(160, 82, 45) }, 
  { wxT("silver"), wxRGBColor(192, 192, 192) }, 
  { wxT("skyblue"), wxRGBColor(135, 206, 235) }, 
  { wxT("slateblue"), wxRGBColor(106, 90, 205) }, 
  { wxT("slategray"), wxRGBColor(112, 128, 144) }, 
  { wxT("slategrey"), wxRGBColor(112, 128, 144) }, 
  { wxT("snow"), wxRGBColor(255, 250, 250) }, 
  { wxT("springgreen"), wxRGBColor(0, 255, 127) }, 
  { wxT("steelblue"), wxRGBColor(70, 130, 180) }, 
  { wxT("tan"), wxRGBColor(210, 180, 140) }, 
  { wxT("teal"), wxRGBColor(0, 128, 128) }, 
  { wxT("thistle"), wxRGBColor(216, 191, 216) }, 
  { wxT("tomato"), wxRGBColor(255, 99, 71) }, 
  { wxT("turquoise"), wxRGBColor(64, 224, 208) }, 
  { wxT("violet"), wxRGBColor(238, 130, 238) }, 
  { wxT("wheat"), wxRGBColor(245, 222, 179) }, 
  { wxT("white"), wxRGBColor(255, 255, 255) }, 
  { wxT("whitesmoke"), wxRGBColor(245, 245, 245) }, 
  { wxT("yellow"), wxRGBColor(255, 255, 0) }, 
  { wxT("yellowgreen"), wxRGBColor(154, 205, 50) } 
}; 

