//////////////////////////////////////////////////////////////////////////////
// Name:        SVGElementInstanceList.cpp
// Author:      Alex Thuering
// Copyright:   (c) 2005 Alex Thuering
// Licence:     wxWindows licence
// Notes:       generated by genList.py
//////////////////////////////////////////////////////////////////////////////

#include "SVGElementInstanceList.h"
#include <wx/arrimpl.cpp>
WX_DEFINE_OBJARRAY(wxSVGElementInstanceList);
