//////////////////////////////////////////////////////////////////////////////
// Name:        css_values.cpp
// Author:      Alex Thuering
// Copyright:   (c) 2005 Alex Thuering
// Licence:     wxWindows licence
// Notes:       generated by genCSS.py
//////////////////////////////////////////////////////////////////////////////

wxString s_cssValueStrings[] =
{
  wxT("100"),
  wxT("200"),
  wxT("300"),
  wxT("400"),
  wxT("500"),
  wxT("600"),
  wxT("700"),
  wxT("800"),
  wxT("900"),
  wxT("accumulate"),
  wxT("after-edge"),
  wxT("all"),
  wxT("alphabetic"),
  wxT("auto"),
  wxT("baseline"),
  wxT("before-edge"),
  wxT("bevel"),
  wxT("bidi-override"),
  wxT("blink"),
  wxT("block"),
  wxT("bold"),
  wxT("bolder"),
  wxT("butt"),
  wxT("central"),
  wxT("collapse"),
  wxT("compact"),
  wxT("condensed"),
  wxT("crispEdges"),
  wxT("crosshair"),
  wxT("currentColor"),
  wxT("default"),
  wxT("e-resize"),
  wxT("embed"),
  wxT("end"),
  wxT("evenodd"),
  wxT("expanded"),
  wxT("extra-condensed"),
  wxT("extra-expanded"),
  wxT("fill"),
  wxT("geometricPrecision"),
  wxT("hanging"),
  wxT("help"),
  wxT("hidden"),
  wxT("ideographic"),
  wxT("inline"),
  wxT("inline-table"),
  wxT("italic"),
  wxT("lighter"),
  wxT("line-through"),
  wxT("linearRGB"),
  wxT("list-item"),
  wxT("lr"),
  wxT("lr-tb"),
  wxT("ltr"),
  wxT("marker"),
  wxT("mathematical"),
  wxT("middle"),
  wxT("miter"),
  wxT("move"),
  wxT("n-resize"),
  wxT("narrower"),
  wxT("ne-resize"),
  wxT("new"),
  wxT("no-change"),
  wxT("none"),
  wxT("nonzero"),
  wxT("normal"),
  wxT("nw-resize"),
  wxT("oblique"),
  wxT("optimizeLegibility"),
  wxT("optimizeQuality"),
  wxT("optimizeSpeed"),
  wxT("overline"),
  wxT("painted"),
  wxT("pointer"),
  wxT("reset-size"),
  wxT("rl"),
  wxT("rl-tb"),
  wxT("round"),
  wxT("rtl"),
  wxT("run-in"),
  wxT("s-resize"),
  wxT("sRGB"),
  wxT("scroll"),
  wxT("se-resize"),
  wxT("semi-condensed"),
  wxT("semi-expanded"),
  wxT("small-caps"),
  wxT("square"),
  wxT("start"),
  wxT("stroke"),
  wxT("sub"),
  wxT("super"),
  wxT("sw-resize"),
  wxT("table"),
  wxT("table-caption"),
  wxT("table-cell"),
  wxT("table-column"),
  wxT("table-column-group"),
  wxT("table-footer-group"),
  wxT("table-header-group"),
  wxT("table-row"),
  wxT("table-row-group"),
  wxT("tb"),
  wxT("tb-rl"),
  wxT("text"),
  wxT("text-after-edge"),
  wxT("text-before-edge"),
  wxT("ultra-condensed"),
  wxT("ultra-expanded"),
  wxT("underline"),
  wxT("use-script"),
  wxT("visible"),
  wxT("visibleFill"),
  wxT("visiblePainted"),
  wxT("visibleStroke"),
  wxT("w-resize"),
  wxT("wait"),
  wxT("wider")
};
